@@ -99,11 +99,11 @@ extract_initials_from_text (const char *text)
   g_string_append_unichar (initials, unichar);
 
   q = g_utf8_strrchr (normalized, -1, ' ');
-  if (q != NULL && g_utf8_next_char (q) != NULL) {
-    q = g_utf8_next_char (q);
+  if (q != NULL) {
+    unichar = g_utf8_get_char (g_utf8_next_char (q));
 
-    unichar = g_utf8_get_char (q);
-    g_string_append_unichar (initials, unichar);
+    if (unichar != 0)
+      g_string_append_unichar (initials, unichar);
   }
 
   g_free (normalized);
@@ -330,11 +330,9 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * Since: 1.0
    */
   props[PROP_ICON_NAME] =
-    g_param_spec_string ("icon-name",
-                         "Icon name",
-                         "The name of an icon to use as a fallback",
+    g_param_spec_string ("icon-name", NULL, NULL,
                          NULL,
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwAvatar:text: (attributes org.gtk.Property.get=adw_avatar_get_text org.gtk.Property.set=adw_avatar_set_text)
@@ -347,11 +345,9 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * Since: 1.0
    */
   props[PROP_TEXT] =
-    g_param_spec_string ("text",
-                         "Text",
-                         "The text used to generate the color and the initials",
+    g_param_spec_string ("text", NULL, NULL,
                          "",
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwAvatar:show-initials: (attributes org.gtk.Property.get=adw_avatar_get_show_initials org.gtk.Property.set=adw_avatar_set_show_initials)
@@ -363,11 +359,9 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * Since: 1.0
    */
   props[PROP_SHOW_INITIALS] =
-    g_param_spec_boolean ("show-initials",
-                          "Show initials",
-                          "Whether initials are used instead of an icon on the fallback avatar",
+    g_param_spec_boolean ("show-initials", NULL, NULL,
                           FALSE,
-                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwAvatar:custom-image: (attributes org.gtk.Property.get=adw_avatar_get_custom_image org.gtk.Property.set=adw_avatar_set_custom_image)
@@ -377,11 +371,9 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * Since: 1.0
    */
   props[PROP_CUSTOM_IMAGE] =
-    g_param_spec_object ("custom-image",
-                         "Custom image",
-                         "A custom image to use instead of initials or icon",
+    g_param_spec_object ("custom-image", NULL, NULL,
                          GDK_TYPE_PAINTABLE,
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwAvatar:size: (attributes org.gtk.Property.get=adw_avatar_get_size org.gtk.Property.set=adw_avatar_set_size)
@@ -391,11 +383,9 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * Since: 1.0
    */
   props[PROP_SIZE] =
-    g_param_spec_int ("size",
-                      "Size",
-                      "The size of the avatar",
+    g_param_spec_int ("size", NULL, NULL,
                       -1, INT_MAX, -1,
-                      G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, PROP_LAST_PROP, props);
 
@@ -661,6 +651,7 @@ adw_avatar_set_custom_image (AdwAvatar    *self,
     gtk_widget_remove_css_class (self->gizmo, "image");
   }
 
+  update_initials (self);
   update_visibility (self);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_CUSTOM_IMAGE]);
@@ -742,14 +733,20 @@ adw_avatar_draw_to_texture (AdwAvatar *self,
   GtkNative *native;
   GskRenderer *renderer;
   int size;
+  graphene_matrix_t transform;
 
   g_return_val_if_fail (ADW_IS_AVATAR (self), NULL);
   g_return_val_if_fail (scale_factor > 0, NULL);
 
   size = self->size * scale_factor;
 
+  g_assert (gtk_widget_compute_transform (GTK_WIDGET (self),
+                                          self->gizmo,
+                                          &transform));
+
   snapshot = gtk_snapshot_new ();
   gtk_snapshot_scale (snapshot, scale_factor, scale_factor);
+  gtk_snapshot_transform_matrix (snapshot, &transform);
   GTK_WIDGET_GET_CLASS (self)->snapshot (GTK_WIDGET (self), snapshot);
 
   node = gtk_snapshot_free_to_node (snapshot);
