@@ -61,6 +61,7 @@ struct _AdwStyleManager
   gboolean dark;
   gboolean setting_dark;
 
+  GtkCssProvider *animations_provider;
   guint animation_timeout_id;
 };
 
@@ -95,7 +96,7 @@ warn_prefer_dark_theme (AdwStyleManager *self)
 static void
 unregister_display (GdkDisplay *display)
 {
-  g_assert (!g_hash_table_contains (display_style_managers, display));
+  g_assert (g_hash_table_contains (display_style_managers, display));
 
   g_hash_table_remove (display_style_managers, display);
 }
@@ -123,8 +124,8 @@ register_display (GdkDisplayManager *display_manager,
 static gboolean
 enable_animations_cb (AdwStyleManager *self)
 {
-  gtk_settings_reset_property (gtk_settings_get_for_display (self->display),
-                               "gtk-enable-animations");
+  gtk_style_context_remove_provider_for_display (self->display,
+                                                 GTK_STYLE_PROVIDER (self->animations_provider));
 
   self->animation_timeout_id = 0;
 
@@ -135,24 +136,18 @@ static void
 update_stylesheet (AdwStyleManager *self)
 {
   GtkSettings *gtk_settings;
-  gboolean enable_animations;
 
   if (!self->display)
     return;
 
   gtk_settings = gtk_settings_get_for_display (self->display);
 
-  if (self->animation_timeout_id) {
+  if (self->animation_timeout_id)
     g_clear_handle_id (&self->animation_timeout_id, g_source_remove);
-    enable_animations = TRUE;
-  } else {
-    g_object_get (gtk_settings,
-                  "gtk-enable-animations", &enable_animations,
-                  NULL);
-  }
 
-  if (enable_animations)
-    g_object_set (gtk_settings, "gtk-enable-animations", FALSE, NULL);
+  gtk_style_context_add_provider_for_display (self->display,
+                                              GTK_STYLE_PROVIDER (self->animations_provider),
+                                              10000);
 
   self->setting_dark = TRUE;
 
@@ -180,12 +175,10 @@ update_stylesheet (AdwStyleManager *self)
                                            "/org/gnome/Adwaita/styles/defaults-light.css");
   }
 
-  if (enable_animations) {
-    self->animation_timeout_id =
-      g_timeout_add (SWITCH_DURATION,
-                     G_SOURCE_FUNC (enable_animations_cb),
-                     self);
-  }
+  self->animation_timeout_id =
+    g_timeout_add (SWITCH_DURATION,
+                   G_SOURCE_FUNC (enable_animations_cb),
+                   self);
 }
 
 static gboolean
@@ -279,6 +272,11 @@ adw_style_manager_constructed (GObject *object)
                                                   GTK_STYLE_PROVIDER (self->colors_provider),
                                                   GTK_STYLE_PROVIDER_PRIORITY_THEME);
     }
+
+    self->animations_provider = gtk_css_provider_new ();
+    gtk_css_provider_load_from_data (self->animations_provider,
+                                     "* { transition: none; }",
+                                     -1);
   }
 
   self->settings = adw_settings_get_default ();
@@ -311,6 +309,7 @@ adw_style_manager_dispose (GObject *object)
   g_clear_handle_id (&self->animation_timeout_id, g_source_remove);
   g_clear_object (&self->provider);
   g_clear_object (&self->colors_provider);
+  g_clear_object (&self->animations_provider);
 
   G_OBJECT_CLASS (adw_style_manager_parent_class)->dispose (object);
 }
@@ -392,11 +391,9 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * Since: 1.0
    */
   props[PROP_DISPLAY] =
-    g_param_spec_object ("display",
-                         "Display",
-                         "The display the style manager is associated with",
+    g_param_spec_object ("display", NULL, NULL,
                          GDK_TYPE_DISPLAY,
-                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   /**
    * AdwStyleManager:color-scheme: (attributes org.gtk.Property.get=adw_style_manager_get_color_scheme org.gtk.Property.set=adw_style_manager_set_color_scheme)
@@ -436,12 +433,10 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * Since: 1.0
    */
   props[PROP_COLOR_SCHEME] =
-    g_param_spec_enum ("color-scheme",
-                       "Color Scheme",
-                       "The current color scheme",
+    g_param_spec_enum ("color-scheme", NULL, NULL,
                        ADW_TYPE_COLOR_SCHEME,
                        ADW_COLOR_SCHEME_DEFAULT,
-                       G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwStyleManager:system-supports-color-schemes: (attributes org.gtk.Property.get=adw_style_manager_get_system_supports_color_schemes)
@@ -457,11 +452,9 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * Since: 1.0
    */
   props[PROP_SYSTEM_SUPPORTS_COLOR_SCHEMES] =
-    g_param_spec_boolean ("system-supports-color-schemes",
-                          "System supports color schemes",
-                          "Whether the system supports color schemes",
+    g_param_spec_boolean ("system-supports-color-schemes", NULL, NULL,
                           FALSE,
-                          G_PARAM_READABLE);
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
    * AdwStyleManager:dark: (attributes org.gtk.Property.get=adw_style_manager_get_dark)
@@ -474,11 +467,9 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * Since: 1.0
    */
   props[PROP_DARK] =
-    g_param_spec_boolean ("dark",
-                          "Dark",
-                          "Whether the application is using dark appearance",
+    g_param_spec_boolean ("dark", NULL, NULL,
                           FALSE,
-                          G_PARAM_READABLE);
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
    * AdwStyleManager:high-contrast: (attributes org.gtk.Property.get=adw_style_manager_get_high_contrast)
@@ -490,11 +481,9 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * Since: 1.0
    */
   props[PROP_HIGH_CONTRAST] =
-    g_param_spec_boolean ("high-contrast",
-                          "High Contrast",
-                          "Whether the application is using high contrast appearance",
+    g_param_spec_boolean ("high-contrast", NULL, NULL,
                           FALSE,
-                          G_PARAM_READABLE);
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 }
