@@ -10,6 +10,8 @@
 
 #include "adw-macros-private.h"
 
+#include <stdarg.h>
+
 /**
  * AdwToastPriority:
  * @ADW_TOAST_PRIORITY_NORMAL: the toast will be queued if another toast is
@@ -31,7 +33,7 @@
  * follows:
  *
  * ```c
- * adw_toast_overlay_add_toast (overlay, adw_toast_new (_("Simple Toast"));
+ * adw_toast_overlay_add_toast (overlay, adw_toast_new (_("Simple Toast")));
  * ```
  *
  * <picture>
@@ -46,6 +48,9 @@
  * [property@Toast:priority] determines how it behaves if another toast is
  * already being displayed.
  *
+ * [property@Toast:custom-title] can be used to replace the title label with a
+ * custom widget.
+ *
  * ## Actions
  *
  * Toasts can have one button on them, with a label and an attached
@@ -102,9 +107,7 @@
  *   n_items = ... // The number of waiting items
  *
  *   if (!self->undo_toast) {
- *     title = g_strdup_printf (_("‘%s’ deleted"), ...);
- *
- *     self->undo_toast = adw_toast_new (title);
+ *     self->undo_toast = adw_toast_new_format (_("‘%s’ deleted"), ...);
  *
  *     adw_toast_set_priority (self->undo_toast, ADW_TOAST_PRIORITY_HIGH);
  *     adw_toast_set_button_label (self->undo_toast, _("_Undo"));
@@ -124,6 +127,9 @@
  *                                n_items), n_items);
  *
  *   adw_toast_set_title (self->undo_toast, title);
+ *
+ *   // Bump the toast timeout
+ *   adw_toast_overlay_add_toast (self->toast_overlay, g_object_ref (self->undo_toast));
  * }
  *
  * static void
@@ -152,8 +158,9 @@ struct _AdwToast {
   GVariant *action_target;
   AdwToastPriority priority;
   guint timeout;
+  GtkWidget *custom_title;
 
-  gboolean added;
+  AdwToastOverlay *overlay;
 };
 
 enum {
@@ -164,6 +171,7 @@ enum {
   PROP_ACTION_TARGET,
   PROP_PRIORITY,
   PROP_TIMEOUT,
+  PROP_CUSTOM_TITLE,
   LAST_PROP,
 };
 
@@ -171,6 +179,7 @@ static GParamSpec *props[LAST_PROP];
 
 enum {
   SIGNAL_DISMISSED,
+  SIGNAL_BUTTON_CLICKED,
   SIGNAL_LAST_SIGNAL,
 };
 
@@ -181,7 +190,17 @@ G_DEFINE_FINAL_TYPE (AdwToast, adw_toast, G_TYPE_OBJECT)
 static void
 dismissed_cb (AdwToast *self)
 {
-  self->added = FALSE;
+  adw_toast_set_overlay (self, NULL);
+}
+
+static void
+adw_toast_dispose (GObject *object)
+{
+  AdwToast *self = ADW_TOAST (object);
+
+  g_clear_object (&self->custom_title);
+
+  G_OBJECT_CLASS (adw_toast_parent_class)->dispose (object);
 }
 
 static void
@@ -224,6 +243,9 @@ adw_toast_get_property (GObject    *object,
   case PROP_TIMEOUT:
     g_value_set_uint (value, adw_toast_get_timeout (self));
     break;
+  case PROP_CUSTOM_TITLE:
+    g_value_set_object (value, adw_toast_get_custom_title (self));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -256,6 +278,9 @@ adw_toast_set_property (GObject      *object,
   case PROP_TIMEOUT:
     adw_toast_set_timeout (self, g_value_get_uint (value));
     break;
+  case PROP_CUSTOM_TITLE:
+    adw_toast_set_custom_title (self, g_value_get_object (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -266,6 +291,7 @@ adw_toast_class_init (AdwToastClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->dispose = adw_toast_dispose;
   object_class->finalize = adw_toast_finalize;
   object_class->get_property = adw_toast_get_property;
   object_class->set_property = adw_toast_set_property;
@@ -277,14 +303,16 @@ adw_toast_class_init (AdwToastClass *klass)
    *
    * The title can be marked up with the Pango text markup language.
    *
+   * Setting a title will unset [property@Toast:custom-title].
+   *
+   * If [property@Toast:custom-title] is set, it will be used instead.
+   *
    * Since: 1.0
    */
   props[PROP_TITLE] =
-    g_param_spec_string ("title",
-                         "Title",
-                         "The title of the toast",
+    g_param_spec_string ("title", NULL, NULL,
                          "",
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwToast:button-label: (attributes org.gtk.Property.get=adw_toast_get_button_label org.gtk.Property.set=adw_toast_set_button_label)
@@ -300,11 +328,9 @@ adw_toast_class_init (AdwToastClass *klass)
    * Since: 1.0
    */
   props[PROP_BUTTON_LABEL] =
-    g_param_spec_string ("button-label",
-                         "Button Label",
-                         "The label to show on the button",
+    g_param_spec_string ("button-label", NULL, NULL,
                          NULL,
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwToast:action-name: (attributes org.gtk.Property.get=adw_toast_get_action_name org.gtk.Property.set=adw_toast_set_action_name)
@@ -318,11 +344,9 @@ adw_toast_class_init (AdwToastClass *klass)
    * Since: 1.0
    */
   props[PROP_ACTION_NAME] =
-    g_param_spec_string ("action-name",
-                         "Action Name",
-                         "The name of the associated action",
+    g_param_spec_string ("action-name", NULL, NULL,
                          NULL,
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwToast:action-target: (attributes org.gtk.Property.get=adw_toast_get_action_target_value org.gtk.Property.set=adw_toast_set_action_target_value)
@@ -334,12 +358,10 @@ adw_toast_class_init (AdwToastClass *klass)
    * Since: 1.0
    */
   props[PROP_ACTION_TARGET] =
-    g_param_spec_variant ("action-target",
-                          "Action Target Value",
-                          "The parameter for action invocations",
+    g_param_spec_variant ("action-target", NULL, NULL,
                           G_VARIANT_TYPE_ANY,
                           NULL,
-                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwToast:priority: (attributes org.gtk.Property.get=adw_toast_get_priority org.gtk.Property.set=adw_toast_set_priority)
@@ -357,12 +379,10 @@ adw_toast_class_init (AdwToastClass *klass)
    * Since: 1.0
    */
   props[PROP_PRIORITY] =
-    g_param_spec_enum ("priority",
-                       "Priority",
-                       "The priority of the toast",
+    g_param_spec_enum ("priority", NULL, NULL,
                        ADW_TYPE_TOAST_PRIORITY,
                        ADW_TOAST_PRIORITY_NORMAL,
-                       G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwToast:timeout: (attributes org.gtk.Property.get=adw_toast_get_timeout org.gtk.Property.set=adw_toast_set_timeout)
@@ -378,11 +398,26 @@ adw_toast_class_init (AdwToastClass *klass)
    * Since: 1.0
    */
   props[PROP_TIMEOUT] =
-    g_param_spec_uint ("timeout",
-                       "Timeout",
-                       "The timeout of the toast, in seconds",
+    g_param_spec_uint ("timeout", NULL, NULL,
                        0, G_MAXUINT, 5,
-                       G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwToast:custom-title: (attributes org.gtk.Property.get=adw_toast_get_custom_title org.gtk.Property.set=adw_toast_set_custom_title)
+   *
+   * The custom title widget.
+   *
+   * It will be displayed instead of the title if set. In this case,
+   * [property@Toast:title] is ignored.
+   *
+   * Setting a custom title will unset [property@Toast:title].
+   *
+   * Since: 1.2
+   */
+  props[PROP_CUSTOM_TITLE] =
+    g_param_spec_object ("custom-title", NULL, NULL,
+                         GTK_TYPE_WIDGET,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
@@ -400,6 +435,23 @@ adw_toast_class_init (AdwToastClass *klass)
                   0,
                   NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
+
+  /**
+   * AdwToast::button-clicked:
+   *
+   * Emitted after the button has been clicked.
+   *
+   * It can be used as an alternative to setting an action.
+   *
+   * Since: 1.2
+   */
+  signals[SIGNAL_BUTTON_CLICKED] =
+    g_signal_new ("button-clicked",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
 }
 
 static void
@@ -408,6 +460,7 @@ adw_toast_init (AdwToast *self)
   self->title = g_strdup ("");
   self->priority = ADW_TOAST_PRIORITY_NORMAL;
   self->timeout = 5;
+  self->custom_title = NULL;
 
   g_signal_connect (self, "dismissed", G_CALLBACK (dismissed_cb), self);
 }
@@ -436,13 +489,52 @@ adw_toast_new (const char *title)
                        NULL);
 }
 
+/**
+ * adw_toast_new_format:
+ * @format: the formatted string for the toast title
+ * @...: the parameters to insert into the format string
+ *
+ * Creates a new `AdwToast`.
+ *
+ * The toast will use the format string as its title.
+ *
+ * See also: [ctor@Toast.new]
+ *
+ * Returns: the newly created toast object
+ *
+ * Since: 1.2
+ */
+AdwToast *
+adw_toast_new_format (const char *format,
+                      ...)
+{
+  AdwToast *res;
+  va_list args;
+  char *title;
+
+  va_start (args, format);
+  title = g_strdup_vprintf (format, args);
+  va_end (args);
+
+  res = g_object_new (ADW_TYPE_TOAST,
+                      "title", title,
+                      NULL);
+
+  g_free (title);
+
+  return res;
+}
+
 /**
  * adw_toast_get_title: (attributes org.gtk.Method.get_property=title)
  * @self: a toast
  *
  * Gets the title that will be displayed on the toast.
  *
- * Returns: the title
+ * If a custom title has been set with [method@Adw.Toast.set_custom_title]
+ * the return value will be %NULL.
+ *
+ * Returns: (nullable): the title
  *
  * Since: 1.0
  */
@@ -451,7 +543,10 @@ adw_toast_get_title (AdwToast *self)
 {
   g_return_val_if_fail (ADW_IS_TOAST (self), NULL);
 
-  return self->title;
+  if (self->custom_title == NULL)
+    return self->title;
+
+  return NULL;
 }
 
 /**
@@ -473,10 +568,16 @@ adw_toast_set_title (AdwToast   *self,
   if (!g_strcmp0 (self->title, title))
     return;
 
+  g_object_freeze_notify (G_OBJECT (self));
+
+  adw_toast_set_custom_title (self, NULL);
+
   g_clear_pointer (&self->title, g_free);
   self->title = g_strdup (title);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TITLE]);
+
+  g_object_thaw_notify (G_OBJECT (self));
 }
 
 /**
@@ -590,6 +691,9 @@ adw_toast_get_action_target_value (AdwToast *self)
  *
  * Sets the parameter for action invocations.
  *
+ * If the @action_target variant has a floating reference this function
+ * will sink it.
+ *
  * Since: 1.0
  */
 void
@@ -606,7 +710,8 @@ adw_toast_set_action_target_value (AdwToast *self,
     return;
 
   g_clear_pointer (&self->action_target, g_variant_unref);
-  self->action_target = g_variant_ref (action_target);
+  if (action_target != NULL)
+    self->action_target = g_variant_ref_sink (action_target);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ACTION_TARGET]);
 }
@@ -788,6 +893,9 @@ adw_toast_set_timeout (AdwToast *self,
  *
  * Dismisses @self.
  *
+ * Does nothing if @self has already been dismissed, or hasn't been added to an
+ * [class@ToastOverlay].
+ *
  * Since: 1.0
  */
 void
@@ -795,29 +903,74 @@ adw_toast_dismiss (AdwToast *self)
 {
   g_return_if_fail (ADW_IS_TOAST (self));
 
-  if (!self->added) {
-    g_critical ("Trying to dismiss the toast '%s', but it isn't in an "
-                "AdwToastOverlay yet", self->title);
-
+  if (!self->overlay)
     return;
-  }
 
   g_signal_emit (self, signals[SIGNAL_DISMISSED], 0, NULL);
 }
 
-gboolean
-adw_toast_get_added (AdwToast *self)
+/**
+ * adw_toast_set_custom_title: (attributes org.gtk.Method.set_property=custom-title)
+ * @self: a toast
+ * @widget: (nullable): the custom title widget
+ *
+ * Sets the custom title widget of @self.
+ *
+ * Since: 1.2
+ */
+void
+adw_toast_set_custom_title (AdwToast  *self,
+                            GtkWidget *widget)
 {
-  g_return_val_if_fail (ADW_IS_TOAST (self), FALSE);
+  g_return_if_fail (ADW_IS_TOAST (self));
+  g_return_if_fail (widget == NULL || GTK_IS_WIDGET (widget));
 
-  return self->added;
+  if (self->custom_title == widget)
+    return;
+
+  g_object_freeze_notify (G_OBJECT (self));
+
+  adw_toast_set_title (self, "");
+
+  g_set_object (&self->custom_title, widget);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_CUSTOM_TITLE]);
+
+  g_object_thaw_notify (G_OBJECT (self));
+}
+
+/**
+ * adw_toast_get_custom_title: (attributes org.gtk.Method.get_property=custom-title)
+ * @self: a toast
+ *
+ * Gets the custom title widget of @self.
+ *
+ * Returns: (nullable) (transfer none): the custom title widget
+ *
+ * Since: 1.2
+ */
+GtkWidget *
+adw_toast_get_custom_title (AdwToast *self)
+{
+  g_return_val_if_fail (ADW_IS_TOAST (self), NULL);
+
+  return self->custom_title;
+}
+
+AdwToastOverlay *
+adw_toast_get_overlay (AdwToast *self)
+{
+  g_return_val_if_fail (ADW_IS_TOAST (self), NULL);
+
+  return self->overlay;
 }
 
 void
-adw_toast_set_added (AdwToast *self,
-                     gboolean  added)
+adw_toast_set_overlay (AdwToast        *self,
+                       AdwToastOverlay *overlay)
 {
   g_return_if_fail (ADW_IS_TOAST (self));
+  g_return_if_fail (overlay == NULL || ADW_IS_TOAST_OVERLAY (overlay));
 
-  self->added = !!added;
+  self->overlay = overlay;
 }
