@@ -43,17 +43,17 @@
  *
  * ```xml
  * <object class="GtkWindow">
- *   <child type="titlebar">
+ *   <property name="titlebar">
  *     <object class="AdwHeaderBar">
  *       <property name="centering-policy">strict</property>
- *       <child type="title">
+ *       <property name="title-widget">
  *         <object class="AdwViewSwitcherTitle" id="title">
  *           <property name="stack">stack</property>
  *         </object>
  *       </child>
  *     </object>
- *   </child>
- *   <child>
+ *   </property>
+ *   <property>
  *     <object class="GtkBox">
  *       <property name="orientation">vertical</property>
  *       <child>
@@ -68,7 +68,7 @@
  *         </object>
  *       </child>
  *     </object>
- *   </child>
+ *   </property>
  * </object>
  * ```
  *
@@ -102,6 +102,8 @@ struct _AdwViewSwitcherTitle
   gboolean is_window_narrow;
 
   GtkSelectionModel *pages;
+
+  guint check_window_width_id;
 };
 
 static GParamSpec *props[LAST_PROP];
@@ -126,6 +128,8 @@ update_view_switcher_visible (AdwViewSwitcherTitle *self)
 
       if (adw_view_stack_page_get_visible (page))
         count++;
+
+      g_object_unref (page);
     }
   }
 
@@ -222,13 +226,15 @@ check_window_width (AdwViewSwitcherTitle *self)
   self->is_window_narrow = width <= 360;
   update_view_switcher_visible (self);
 
+  self->check_window_width_id = 0;
   return G_SOURCE_REMOVE;
 }
 
 static void
 notify_surface_width_cb (AdwViewSwitcherTitle *self)
 {
-  g_idle_add (G_SOURCE_FUNC (check_window_width), self);
+  if (self->check_window_width_id == 0)
+    self->check_window_width_id = g_idle_add (G_SOURCE_FUNC (check_window_width), self);
 }
 
 static void
@@ -256,6 +262,8 @@ adw_view_switcher_title_unrealize (GtkWidget *widget)
 
   g_signal_handlers_disconnect_by_func (surface, notify_surface_width_cb, self);
 
+  g_clear_handle_id (&self->check_window_width_id, g_source_remove);
+
   GTK_WIDGET_CLASS (adw_view_switcher_title_parent_class)->unrealize (widget);
 }
 
@@ -280,11 +288,9 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    * Since: 1.0
    */
   props[PROP_STACK] =
-    g_param_spec_object ("stack",
-                         "Stack",
-                         "The stack the view switcher controls",
+    g_param_spec_object ("stack", NULL, NULL,
                          ADW_TYPE_VIEW_STACK,
-                         G_PARAM_EXPLICIT_NOTIFY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwViewSwitcherTitle:title: (attributes org.gtk.Property.get=adw_view_switcher_title_get_title org.gtk.Property.set=adw_view_switcher_title_set_title)
@@ -297,11 +303,9 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    * Since: 1.0
    */
   props[PROP_TITLE] =
-    g_param_spec_string ("title",
-                         "Title",
-                         "The title to display",
+    g_param_spec_string ("title", NULL, NULL,
                          "",
-                         G_PARAM_EXPLICIT_NOTIFY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwViewSwitcherTitle:subtitle: (attributes org.gtk.Property.get=adw_view_switcher_title_get_subtitle org.gtk.Property.set=adw_view_switcher_title_set_subtitle)
@@ -313,11 +317,9 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    * Since: 1.0
    */
   props[PROP_SUBTITLE] =
-    g_param_spec_string ("subtitle",
-                         "Subtitle",
-                         "The subtitle to display",
+    g_param_spec_string ("subtitle", NULL, NULL,
                          "",
-                         G_PARAM_EXPLICIT_NOTIFY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwViewSwitcherTitle:view-switcher-enabled: (attributes org.gtk.Property.get=adw_view_switcher_title_get_view_switcher_enabled org.gtk.Property.set=adw_view_switcher_title_set_view_switcher_enabled)
@@ -334,11 +336,9 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    * Since: 1.0
    */
   props[PROP_VIEW_SWITCHER_ENABLED] =
-    g_param_spec_boolean ("view-switcher-enabled",
-                         "View switcher enabled",
-                         "Whether the view switcher is enabled",
+    g_param_spec_boolean ("view-switcher-enabled", NULL, NULL,
                          TRUE,
-                         G_PARAM_EXPLICIT_NOTIFY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwViewSwitcherTitle:title-visible: (attributes org.gtk.Property.get=adw_view_switcher_title_get_title_visible)
@@ -351,11 +351,9 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    * Since: 1.0
    */
   props[PROP_TITLE_VISIBLE] =
-    g_param_spec_boolean ("title-visible",
-                         "Title visible",
-                         "Whether the title is currently visible",
+    g_param_spec_boolean ("title-visible", NULL, NULL,
                          TRUE,
-                         G_PARAM_EXPLICIT_NOTIFY | G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
